//
// Post Example for the Registry Compliance API.
// Copyright (c) 2003-2006 by Unspam Registry Services, Inc.
// All Rights Reserved.
// 
// You may use these Code Samples for any purpose, commercial or otherwise, provided that you 
// adhere to the disclaimer at the bottom of this document. You must also require any transferee of this 
// Code Sample to abide by the terms of the disclaimer.
// 


using System;
using System.IO;
using System.Net;
using System.Text;

protected void ExecuteQuery(string serverID, string taskKey, string dataType, string filePath)
{
	// Constants
	const string	API_KEY				= "Your API Key Goes Here";
	const string	ACCOUNT_KEY			= "&account_key=Your Account Key Goes Here";

	const string	INET_OBJECT			= "/serve.html";			
	const string	INET_SERVER			= "https://api.registrycompliance.com" + INET_OBJECT;			
	const string	INET_AGENT			= "RegistryComplianceAPIAccess";			
	const string	INET_USERNAME		= "";			
	const string	INET_PASSWORD		= "";			

	const int		TIMEOUT				= 20000;

	const string	CONTENT_BOUNDARY	= "YourBoundaryGoesHere";
	const string	CONTENT_TYPE		= "multipart/form-data; boundary=" + CONTENT_BOUNDARY;
	const string	BASE_PARAMS			= "--" + CONTENT_BOUNDARY + "\r\nContent-Disposition: form-data; name=\"request_params\"\r\n\r\napi_key=";

	const string	OP_TASK_ADD			= "&op=TASK_ADD";				

	const string	TASK_ADD_FILE		= "Content-Disposition: form-data; name=\"datafile\"; filename=\"datafile\"\r\nContent-Type: application/octet-stream";		
	const string	TASK_KEY			= "&task_key=";
	const string	DATA_TYPE			= "&data_type=";
	const string	DATA_SIZE			= "&data_size=";
	const string	DATA_CHECKSUM		= "&data_checksum=";

	const int		READ_BLOCK_SIZE		= 4096;

	// Variables
	string			requestParams		= BASE_PARAMS + API_KEY + ACCOUNT_KEY;
	string	s		erver				= INET_SERVER;
	Stream			postStream			= null;
	WebResponse		response			= null;
	long			finalLength			= 0;
	long			fileOffset			= 0;
	byte[]			buffer				= null;
	ASCIIEncoding	encoding			= new ASCIIEncoding();
	string			endString			= "";
	byte[]			checksum			= null;
				
	using (FileStream fs = File.OpenRead(filePath))
	{
		// Hash the file
		MD5 Hasher = new MD5CryptoServiceProvider();
		checksum = Hasher.ComputeHash(fs);

		postStream = requestParams;
		postStream += OP_TASK_ADD;
		postStream += TASK_KEY;
		postStream += taskKey;
		postStream += DATA_TYPE;
		postStream += dataType;
		postStream += DATA_SIZE;
		postStream += Convert.ToString((fs.Length + checksum.Length));
		postStream += DATA_CHECKSUM;
		postStream += Hex.ToString(checksum);
		postStream += "\r\n--";
		postStream += CONTENT_BOUNDARY;
		postStream += "\r\n";
		postStream += TASK_ADD_FILE;
		postStream += "\r\n\r\n";

		fs.Close();
	}
	
	// Create the string information that follows the file in the POST
	endString = "\r\n";
	endString += "--";
	endString += CONTENT_BOUNDARY;
	endString += "--";

	// Setup the HttpWebRequest client
	HttpWebRequest client	= (HttpWebRequest) WebRequest.Create(serverID);
	client.ContentType		= CONTENT_TYPE;
	client.UserAgent		= INET_AGENT;
	client.Method			= "POST";
	client.Timeout			= TIMEOUT;
	client.SendChunked		= true;
	
	// Open the file
	using (FileStream fs = File.OpenRead(filePath))
	{
		// Setup a binary file reader to read the file
		BinaryReader br = new BinaryReader(fs);

		// Convert the POST string to binary
		byte[] postSubmission = encoding.GetBytes(postString);
		byte[] endSubmission = encoding.GetBytes(endString);

		// Set the POST content length
		finalLength = (long)(postString.Length + fs.Length + endString.Length);
		client.ContentLength = finalLength;

		// Submit the POST string to the server
		postStream = client.GetRequestStream();	
		postStream.Write(postSubmission, 0, postSubmission.Length);

		// Copy the binary file to the server in chunks 
		buffer = br.ReadBytes(READ_BLOCK_SIZE);
		while (buffer.Length > 0)
		{
			postStream.Write(buffer, 0, buffer.Length);
			fileOffset += buffer.Length;

			bufferPos += READ_BLOCK_SIZE;
	
			buffer = br.ReadBytes(READ_BLOCK_SIZE);
		}

		// Write the POST end string to the server
		postStream.Write(endSubmission, 0, endSubmission.Length);
		postStream.Close();
	}
	
	// Retrieve the Server's Response
	response = client.GetResponse();
	ReadStream(response.GetResponseStream());
	response.Close();
}









// Use of this Code Sample is at Your Own Risk
//
// You understand that your use of the this Code Sample is at your own risk, i.e. Unspam Registry Services, Inc.
// (Unspam) will not be liable either directly or indirectly to you or any third party transferee recipient of the Code Sample.
// Unspam is under no obligation to provide you with any error corrections, updates, upgrades, bug fixes and/or enhancements of the Code Sample. 
//
// This Code Sample May Not Include Adequate System Protection Functionality
//
// The code sample included herein has not been reviewed for suitability in terms of error-handling, system loading, memory requirements,
// CPU usage or other factors that may influence the operation of systems on which it is installed.
//
// Disclaimer of Warranty; Limitation of Liability
// 
// YOUR USE OF THIS CODE SAMPLE IS AT YOUR SOLE RISK. ANY CODE SAMPLE IS PROVIDED "AS IS," "WITH ALL FAULTS" AND "AS AVAILABLE" FOR YOUR USE,
// WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED, UNLESS SUCH WARRANTIES ARE LEGALLY INCAPABLE OF EXCLUSION. SPECIFICALLY, 
// UNSPAM AND ITS VENDORS DISCLAIM IMPLIED WARRANTIES THAT THE CODE SAMPLE IS MERCHANTABLE, OF SATISFACTORY QUALITY, ACCURATE, FIT FOR A PARTICULAR 
// PURPOSE OR NEED, OR NON-INFRINGING. UNSPAM AND ITS VENDORS DO NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE CODE SAMPLE WILL MEET YOUR REQUIREMENTS
// OR THAT THE CODE SAMPLE IS ERROR-FREE, OR THAT DEFECTS IN THE CODE SAMPLE WILL BE CORRECTED. UNSPAM AND ITS VENDORS DO NOT WARRANT OR MAKE ANY
// REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE CODE SAMPLE OR RELATED DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY,
// RELIABILITY OR OTHERWISE. 
