<?php 
//
// Post Example for the Registry Compliance API.
// Copyright (c) 2003-2006 by Unspam Registry Services, Inc.
// All Rights Reserved.
//
// You may use these Code Samples for any purpose, commercial or otherwise, provided that you
// adhere to the disclaimer at the bottom of this document. You must also require any transferee of this
// Code Sample to abide by the terms of the disclaimer.
//
// Requires: PHP 5.0.3+ compiled with OpenSSL, XML, and CLI support
// Run from the command line like so:  php test_markapi.php <TASK> <params>
// 
define('DEBUG_LEVEL',      0); // 0 for no debug, 1 for debug, 2 for more debug (prints binary)
define('PRINT_RESPONSE',   1); // 1 to print the received response

                                                                 
define('REQUEST_PROTOCOL', "https"); // http or https          
define('REQUEST_HOST',     "testapi.registrycompliance.com");  
define('REQUEST_PORT',     "443");                             

define('REQUEST_SCRIPT',   "/serve.html");

// Your direct access API key (96 characters)
define('API_KEY',         "0d8f540449f63c5eb85825d05985aaa075c017c78d11cf7fb9e2b7658b451709584c83bfcac4f57c676f37df6e76dee5");  // (dev sample)

// The session file used to store session internals such as the current task. 
// you should ensure this file is writable.
// If the file does not exist, you should create one [with 0 bytes] at that current path.
define('SESSION_FILE',    "/path/to/session/file");

// For API access, specify either:
//   Your 64 character access key (this key is different from your api key)
define('ACCOUNT_KEY',          "13717b7a6a5e939e3ba9f8968a8e1af8202b96e8e9042b965fe8107a22863d1c"); // (dev sample)

define('BOUNDARY',             "boundary_a92f35f2729ea1e7c21215fd7f72cb6f");

$op       = $_SERVER["argv"][1];
$params   = array();
        
if (!$op) {
    echo "ERROR: No operation specified. (Operation: GET_SALTS,TASK_START,TASK_ADD,TASK_COMMIT,TASK_CANCEL,TASK_LIST,TASK_RESULTS,CLIENT_LIST)\n";echo "Usage: <task> [params]\n";
    exit(1);
}

if (!is_file(SESSION_FILE)) {
    echo "Could not find session file: ".SESSION_FILE."\n";
    exit(2);
} else if (!is_writeable(SESSION_FILE)) {
    echo "The session file does not appear to be writeable: ".SESSION_FILE."\n";
    exit(2);
} else {
    // Load the session data
    $session = unserialize(trim(file_get_contents(SESSION_FILE)));
    if (DEBUG_LEVEL > 0) {
        echo "Session "; print_r($session);
    }
}


$params["api_key"] = API_KEY;
$params["op"]      = strtoupper($op);
if (defined("ACCOUNT_KEY"))  $params["account_key"]  = ACCOUNT_KEY;


switch (strtoupper($params["op"])) {
case "GET_SALTS":
    break;
case "CLIENT_LIST":
    break;
case "TASK_START":
    $params["scrub_juris"] = $_SERVER["argv"][2];
    $params["client_key"]  = $_SERVER["argv"][3];
    if (!$params["scrub_juris"] || !ereg("([0-9]+,?)+",$params["scrub_juris"]) ||
        !$params["client_key"]  || !ereg("^([0-9a-f]{64})\$",$params["client_key"])) {
        echo "ERROR: TASK_START 'scrub_juris' or 'client_key' parameter not specified or invalid.\n";
        echo " * Usage: TASK_START juri1[,juri2,juri3,..] client_key\n";
        exit(2);
    }
    if (isset($session["salta"]) && isset($session["saltb"])) {
        $params["salta"] = $session["salta"];
        $params["saltb"] = $session["saltb"];
    } else {
        echo "ERROR: Salts not found in the session file. Try running a GET_SALTS op first.\n";
        exit(3);
    }
    break;
case "TASK_ADD":
    $usage = "Usage: TASK_ADD <data_type> <data_file> [BIN|HEX]";
    $params["data_type"] = $_SERVER["argv"][2];
    $params["data_file"] = $_SERVER["argv"][3];
    $params["file_upload_format"] = $_SERVER["argv"][4];
    if (!$params["data_type"] || !ereg("^(EML|DMN|FAX|SMS|YIM|ICQ|MSN|AIM){1}\$",$params["data_type"])) {
        echo "ERROR: TASK_ADD 'data_type' parameter not specified or invalid.\n";
        echo " * Usage: $usage\n";
        exit(2);
    }
    if (!$params["data_file"] || !is_file($params["data_file"])) {
        echo "ERROR: TASK_ADD 'data_file' parameter not specified or file was not found.\n";
        echo " * Usage: $usage\n";
        exit(2);
    } else {
        $params["data_size"] = filesize($params["data_file"]);
        if ($params["data_size"] % 16 != 0) {
            echo "ERROR: The data_file size should be divisble by 16.\n";
            echo " * Usage: $usage\n";
            exit(2);
        }
        $params["data_checksum"] = md5_file($params["data_file"]);
    }
    if (isset($session["task_key"])) {
        $params["task_key"] = $session["task_key"];
    } else {
        echo "ERROR: 'task_key' not found in the session file. Try running a TASK_START op first.\n";
        exit(3);
    }
    if (isset($session["task_add_domain"])) {
        $params["task_add_domain"] = $session["task_add_domain"];
    } else {
        echo "ERROR: 'task_add_domain' not found in the session file. Try running a TASK_START op first.\n";
        exit(3);
    }
    break;
case "TASK_COMMIT":
    $usage = "Usage: TASK_COMMIT";
    if (isset($session["task_key"])) {
        $params["task_key"] = $session["task_key"];
    } else {
        echo "ERROR: 'task_key' not found in the session file. Try running a TASK_START op first.\n";
        exit(3);
    }
    break;
case "TASK_CHECK":
    $params["task_key"] = $_SERVER["argv"][2];
    $usage = "Usage: TASK_CHECK [task_key]";
    if (!$params["task_key"]) { 
        if (isset($session["task_key"])) {
            $params["task_key"] = $session["task_key"];
        } else {
            echo "ERROR: 'task_key' not specified on the command line nor found in the session file. Try running a TASK_START op first.\n";
            exit(3);
        }
    }
    break;
case "TASK_RESULTS":
    $params["task_key"]   = $_SERVER["argv"][2];
    $params["result_key"] = $_SERVER["argv"][3];
    $usage = "Usage: TASK_RESULTS [task_key] [result_key]";
    if (!$params["task_key"]) { 
        if (isset($session["task_key"])) {
            $params["task_key"] = $session["task_key"];
        } else {
            echo "ERROR: 'task_key' not specified on the command line nor found in the session file. Try running a TASK_START op first.\n";
            exit(3);
        }
    }
    if (!$params["result_key"]) { 
        if (isset($session["result_key"])) {
            $params["result_key"] = $session["result_key"];
        } else {
            echo "ERROR: 'result_key' not specified on the command line nor found in the session file. Try running a TASK_CHECK op against a finished task first.\n";
            exit(3);
        }
    }
    break;
case "TASK_CANCEL":
    $params["task_key"] = $_SERVER["argv"][2];
    $usage = "Usage: TASK_CANCEL [task_key]";
    if (!$params["task_key"]) { 
        if (isset($session["task_key"])) {
            $params["task_key"] = $session["task_key"];
        } else {
            echo "ERROR: 'task_key' not specified on the command line nor found in the session file.\n";
            exit(3);
        }
    }
    break;
case "TASK_LIST":
    $params["status_to_return"] = $_SERVER["argv"][2];
    $usage = "Usage: TASK_LIST <status_to_return>";
    if (!ereg("^(OPEN|RECENT)\$",$params["status_to_return"])) { 
        echo "ERROR: 'status_to_return' is not valid (must be OPEN/RECENT).\n";
        echo " * Usage: $usage\n";
        exit(3);
    }
    break;
default:
    echo "That operation is not supported.\n";
    exit(4);
}


/**
 * Performs an HTTP or HTTPS multipart file post.
 */
function performRequest($op, & $request, & $session) {
    $response = "";


    if ($op == "TASK_ADD") {
        $host  = isset($session["task_add_domain"]) ? $session["task_add_domain"] : REQUEST_HOST;
    } else {
        $host  = REQUEST_HOST;
    }

    if (DEBUG_LEVEL > 0) {
        echo "CONTACTING HOST: $host ...\n";
    }

    $head  = "POST ".REQUEST_SCRIPT." HTTP/1.1\r\n";
    $head .= "Host: ".$host."\r\n";
    $head .= "User-Agent: TestMarkAPI\r\n";
    $head .= "Content-Type: multipart/form-data; boundary=".BOUNDARY."\r\n";
    $head .= "Content-Length: ".strlen($request)."\r\n";
    $head .= "Connection: close\r\n";
    $head .= "Cache-Control: no-cache\r\n\r\n";

    if (DEBUG_LEVEL > 1) {
        echo "REQUEST:\n";
        echo "$head$request\n";
    }
    
    $errno  = NULL;
    $errstr = NULL;
    $protocol = (REQUEST_PROTOCOL == "https" ? "ssl://" : "");
    $fd = @fsockopen ($protocol.$host, REQUEST_PORT, $errno, $errstr, 30);
    
    if (!$fd) {
        // Socket Error
        die("\n<BR>Unable to contact the Server<BR>\n");
        return "-1";
    } else {
        fputs($fd,$head.$request);
        while(!feof($fd)) {
            $response .= fgets($fd,4096);
        }
        fclose($fd);
    }

    return $response;
}

function getBoundary() {
    return "--".BOUNDARY."\r\n";
}

function getEndBoundary() {
    return "--".BOUNDARY."--\r\n";
}

function prepareRequest(& $params, & $session) {
    $request    = "";
    $request   .= getBoundary();
    $request   .= "Content-Disposition: form-data; name=\"request_params\"\r\n\r\n";
    
    for (reset($params);list($k,$v) = each($params);) {
        $request .= "$k=".urlencode($v)."&";
    }
    $request    = substr($request,0,-1)."\r\n";
    $request   .= isset($params["data_file"]) ? getBoundary() : getEndBoundary();
    

    if (isset($params["data_file"])) {
        $request   .= "Content-Disposition: form-data; name=\"datafile\"; filename=\"datafile\"\r\n";
        $request   .= "Content-Type: application/octet-stream\r\n\r\n";
        $request   .= file_get_contents($params["data_file"]); 
        $request   .= "\r\n";
        $request   .= getEndBoundary();
    }

    return $request;
}


function & xml2obj(& $text) {
    // Isolate the XML from the rest of the response
    $regex = '/<(\w+)[^>]*>(.*?)<\/\\1>/s';
    preg_match_all($regex, $text, $match);
    $xml = & $match[0][0];
    // Return a simplexml object
    return simplexml_load_string($xml);
}


function processResponse(& $response, & $session) {
    $settings  = NULL;
    $xmlobj  = xml2obj($response);

    if (DEBUG_LEVEL > 0) {
        echo "*response*: ";
        print_r($response);
        echo "*xml*: ";
        print_r($xmlobj);
    }

    $op         = strval($xmlobj->REQUEST->OP);
    $result     = strval($xmlobj->RESPONSE->RESULT);
    $errcode    = strval($xmlobj->RESPONSE->ERRCODE);
    $errmsg     = strval($xmlobj->RESPONSE->ERRMSG);
    $salta      = urldecode($xmlobj->RESPONSE->SALTA);
    $saltb      = urldecode($xmlobj->RESPONSE->SALTB);
    $task_key   = strval($xmlobj->RESPONSE->TASK_KEY);
    $result_key = strval($xmlobj->RESPONSE->RESULT_KEY);
    $task_add_domain = strval($xmlobj->RESPONSE->TASK_ADD_DOMAIN);

    if ($salta && $saltb) {
        $session["salta"] = $salta;
        $session["saltb"] = $saltb;
    }
    if ($task_key) {
        $session["task_key"] = $task_key;
    }
    if ($task_add_domain) {
        $session["task_add_domain"] = $task_add_domain;
    }
    if ($op == "TASK_CHECK") {
        if ($result_key) {
            $session["result_key"]  = $result_key;
        }
    }

    if (DEBUG_LEVEL > 0) {
        echo "result:   $result\n";
        echo "errcode:  $errcode\n";
        echo "errmsg:   $errmsg\n";
        echo "salta:    $salta\n";
        echo "saltb:    $saltb\n";
        echo "task_key: $task_key\n";
        echo "task_add_domain: $task_add_domain\n";
    }

    if (PRINT_RESPONSE) {
        echo $response;
    }
}


$response  = "";
$request   = "";
$request   = prepareRequest($params,$session);


$response   = performRequest($params["op"],$request,$session);
if ($response == "-1") {
    exit();
}
processResponse($response,$session);


// Save the session data
if (!($fp = @fopen(SESSION_FILE,"w"))) {
    echo "ERROR: Could not open session file for writing: ".SESSION_FILE."\n";
    exit(10);
}
fwrite($fp,serialize($session));
fclose($fp);

//
// Use of this Code Sample is at Your Own Risk
//
// You understand that your use of the this Code Sample is at your own risk, i.e. Unspam Registry Services, Inc.
// (Unspam) will not be liable either directly or indirectly to you or any third party transferee recipient of the Code Sample.
// Unspam is under no obligation to provide you with any error corrections, updates, upgrades, bug fixes and/or enhancements of the Code Sample.
//
// This Code Sample May Not Include Adequate System Protection Functionality
//
// The code sample included herein has not been reviewed for suitability in terms of error-handling, system loading, memory requirements,
// CPU usage or other factors that may influence the operation of systems on which it is installed.
//
// Disclaimer of Warranty; Limitation of Liability
//
// YOUR USE OF THIS CODE SAMPLE IS AT YOUR SOLE RISK. ANY CODE SAMPLE IS PROVIDED "AS IS," "WITH ALL FAULTS" AND "AS AVAILABLE" FOR YOUR USE,
// WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED, UNLESS SUCH WARRANTIES ARE LEGALLY INCAPABLE OF EXCLUSION. SPECIFICALLY,
// UNSPAM AND ITS VENDORS DISCLAIM IMPLIED WARRANTIES THAT THE CODE SAMPLE IS MERCHANTABLE, OF SATISFACTORY QUALITY, ACCURATE, FIT FOR A PARTICULAR
// PURPOSE OR NEED, OR NON-INFRINGING. UNSPAM AND ITS VENDORS DO NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE CODE SAMPLE WILL MEET YOUR REQUIREMENTS
// OR THAT THE CODE SAMPLE IS ERROR-FREE, OR THAT DEFECTS IN THE CODE SAMPLE WILL BE CORRECTED. UNSPAM AND ITS VENDORS DO NOT WARRANT OR MAKE ANY
// REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE CODE SAMPLE OR RELATED DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY,
// RELIABILITY OR OTHERWISE. 
//


?>
